//+------------------------------------------------------------------+
//|                        Cross NonLag-TEMA with XARD SSA V1.7b.mq4 |
//|                                        Copyright  2010, IND-PC  |
//|                                        http://www.IND-PC.com     |
//+------------------------------------------------------------------+
#property copyright "Aaron Peterson - IND-PC 2010   aaron@ind-pc.com"
#property link      "http://www.ind-pc.com"


extern   double     LotSize = 0.5;
extern   int        TrailingStop=20;
extern   double     MaxDrawDownPct = 2.0;
extern   int        Magic = 1010;
extern   string     TradeComment = "Cross TEMA vs. NonLag 1.7a";
extern   double     CrossDeviation = 0.0005;
extern   int        TEMAPeriod=14;
extern   bool       UseSSA=true;
extern   double     SSABuyLimit = 0.1;
extern   double     SSASellLimit = -0.1;
extern   int     SSALag = 20;
extern   int     SSANumComps = 2;
extern   int     SSAPeriodNorm = 20;
extern   int     SSAN = 300;

static int step = 0;
static double fallBack = 0.0;
static double trailing = 0.0;

string upperComment = "";
double trailingStopWithMultiplier = 0;
static bool buy = false;
static bool sell = false;
//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init()
  {
//----
   
//----  
   return(0);
  }
//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start()
  {
//----
   
//----

upperComment = "";

int ticket, multiplier, found=0;
double orderSize, drawDown;

if(Digits == 2 || Digits == 4)
{
   multiplier = 1;
}
if(Digits == 3 || Digits == 5) 
{
   multiplier = 10;
}
if(Digits == 6) 
{
   multiplier = 100;   
}
if(Digits == 7) 
{
   multiplier = 1000;
}
   
trailingStopWithMultiplier = TrailingStop*multiplier; 

CheckTrailingStop();
CheckFallBack(); 

double NoLag = iCustom(NULL, 0, "NonLagMA", 0,0);
double NoLagPrev = iCustom(NULL, 0, "NonLagMA", 0, 1);
double Tema = iCustom(NULL, 0, "TEMA", TEMAPeriod, 0, 0);
double TemaPrev = iCustom(NULL, 0, "TEMA", TEMAPeriod, 0, 1);
double NoLagUp = iCustom(NULL, 0, "NonLagMA", 1, 0);
double NoLagDown = iCustom(NULL, 0, "NonLagMA", 2, 0);

double SSAValue = 0;
double SSAValuePrev =0; 

if(UseSSA)
{
   SSAValue = iCustom(NULL, 0, "#4X 2011 XARD SSA",SSALag,SSANumComps, SSAPeriodNorm,SSAN, 0, 0);
   SSAValuePrev = iCustom(NULL, 0, "#4X 2011 XARD SSA",SSALag,SSANumComps, SSAPeriodNorm,SSAN, 0, 1);
}


upperComment = "NonLag Value : " + DoubleToStr(NormalizeDouble(NoLag,5),5) + ", Tema Value : " + DoubleToStr(NormalizeDouble(Tema,5),5) + ", SSA Value : " + DoubleToStr(NormalizeDouble(SSAValue,4),4);
Comment(upperComment);



if( IsGreaterThan(Tema, NoLag, CrossDeviation) && !buy && IsGreaterThan(NoLagPrev, TemaPrev, CrossDeviation))//Cross for Buy
{
   found = CloseSellOrders();   
   Print("Done closing SELL items");
   sell = false;
   if(NoLagUp == NoLag && Ask > NoLag && NoLagPrev < NoLag)
   {
      //Buy
      if(UseSSA)
      {
         if(SSAValue >= SSABuyLimit)
         {
            Print("Start Buy with SSAValue above SSABuyLimit");
            BuyCross(found);
            buy = true;
            Print("Done Buy with SSAValue above SSABuyLimit");
         }
         else
         {
            Print("SSAValue not adequate for a buy");
         }
      }
      else
      {
         Print("Start Buy");
         BuyCross(found);
         buy = true;
         Print("Done Buy");
      }
   }   
}
else if(IsGreaterThan(NoLag, Tema, CrossDeviation) && !sell && IsGreaterThan(TemaPrev, NoLagPrev, CrossDeviation))//Cross for Sell
{
   found = CloseBuyOrders();
   Print("Done closing BUY items");
   buy = false;
   if(NoLagDown == NoLag && NoLag > Bid && NoLagPrev > NoLag)
   {
      if(UseSSA)
      {
         if(SSAValue <= SSASellLimit)
         {
            Print("Start Sell with SSAValue below SSASellLimit " + found);
            SellCross(found);
            sell=true;
            Print("Done Buy with SSAValue below SSASellLimit");
         }
      }
      else
      {
         Print("Start Sell");
         SellCross(found);
         sell=true;
         Print("End Sell");
      }
   }
}

if(UseSSA )
{
   if(SSAValue > 0 && SSAValuePrev <=0 && UseSSA) //For SSAVALUE crossing 0
   {
      Print("Crossed 0 Up");
      CloseSellOrders();
      upperComment = upperComment + "\r\n" + "Buy SSA Cross" + " : FallBack = $" + DoubleToStr(NormalizeDouble(fallBack, 2),2);
   }
   else if(SSAValue < 0 && SSAValuePrev >=0 && UseSSA)
   {
      Print("Crossed 0 Down");
      CloseBuyOrders();
      upperComment = upperComment + "\r\n" + "Sell SSA Cross" +  " : FallBack = $" + DoubleToStr(NormalizeDouble(fallBack,2),2);  
   }
}

   drawDown = CheckDrawDown();
   if(drawDown > 0)
   {    
      upperComment = upperComment + "\r\n" + "Current Drawdown : " + DoubleToStr(NormalizeDouble(drawDown,3),3) + "%";
   }

   if(trailing > 0)
   {
      upperComment = upperComment + "\r\n" + "TrailingStop set at: " + DoubleToStr(NormalizeDouble(trailing,5),5);
   }
   
   upperComment = upperComment + "\r\n" + "Buy = " + buy + " Sell = " + sell;
   Comment(upperComment);
   
   
   return(0);
  }
//+------------------------------------------------------------------+

bool IsGreaterThan(double x, double y, double dev)
{
   if(x > y)
   {
      if((x+dev) > y)
      {
         return(true);
      }
   }

   return(false);
}

void CloseOrders()
{
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true && OrderMagicNumber() == Magic)
      {
         if(OrderSymbol()!= Symbol())
         {
            continue;
         }
         int ticket = OrderTicket();
         double orderSize = OrderLots();
            
         switch (OrderType())
         {
            case 0:
               OrderClose(ticket, orderSize, Bid, 2);
               trailing = 0.0;
               Print("Order Closed due to  reaching Trailing Stop");
               break;
                  
            case 1:
               OrderClose(ticket, orderSize, Ask, 2);
               trailing = 0.0;
               Print("Order Closed due to  reaching Trailing Stop");
               break;
         }
         upperComment  = upperComment + " : " + "Close";
      }
   }
}

void CheckFallBack()
{

   for(int i=0; i<OrdersTotal(); i++)
      {
        if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true && OrderMagicNumber() == Magic)
        {
            if(OrderSymbol()== Symbol())
            {
               if(fallBack > 0.0)
               {
                  if(OrderProfit() <= fallBack)
                  {
                     CloseOrders();
                     fallBack = 0.0;
                     return(0);
                  }
               }
              double checkFallBack = (MathFloor(OrderProfit()/(OrderLots() * 100)) - 1) * (OrderLots() * 100);
              if(checkFallBack > fallBack)
              {
               fallBack = checkFallBack;
              }
            }
         }
       }
}

double CheckTrailingStop()
{
      for(int i=0;i<OrdersTotal();i++) 
      {
         OrderSelect(i, SELECT_BY_POS, MODE_TRADES);

         if(OrderType()<=OP_SELL && OrderSymbol()==Symbol() && OrderMagicNumber() == Magic)
         {
            if(OrderType()==OP_BUY)   //<-- Long position is opened
            {
               TrailOrder(OrderType()); return(0); //<-- Trailling the order
            }
            if(OrderType()==OP_SELL) //<-- Go to short position
            {
               TrailOrder(OrderType()); return(0); //<-- Trailling the order
            }
         }
      }
}
void TrailOrder(int type)
{
   if(trailingStopWithMultiplier>0)
   {
      if(OrderMagicNumber() == Magic)
      {
         if(type==OP_BUY)
         {
            if(Bid < trailing)
            {
               CloseOrders();
               Print("Order Closed due to reaching Trailing Stop");
            }
            else if(Bid - (trailingStopWithMultiplier*Point) > trailing)
            {
               trailing = Bid - (trailingStopWithMultiplier*Point);
            }
         }
         if(type==OP_SELL)
         {
            if(Ask > trailing)
            {
               CloseOrders();
               Print("Order Closed due to reaching Trailing Stop");
            }
            else if(Ask + (trailingStopWithMultiplier*Point) < trailing)
            {
               trailing = Ask + (trailingStopWithMultiplier*Point);
            }
         }
      }
   }
}

double CheckDrawDown()
{
   double currentProfit = 0.0;
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true && OrderMagicNumber() == Magic)
      {
         if(OrderSymbol()== Symbol())
         {
            currentProfit = currentProfit + OrderProfit() + OrderCommission() + OrderSwap();
            if((currentProfit / AccountBalance()) * (-100) > MaxDrawDownPct)
            {
               CloseOrders();
               Print("Order Closed due to  reaching Maximum Drawdown");
            }
          }
      }
   }  
   return((currentProfit / AccountBalance()) * (-100));
}

void BuyCross(int found)
{
   if(found==0)
   {
      int tickBuy = OrderSend(Symbol(), OP_BUY, LotSize, Ask, 3, 0, 0,TradeComment, Magic, 0, Blue);
      trailing = Ask - (trailingStopWithMultiplier*Point);
      if(tickBuy < 0)
      {
         Print("Order Buy Failed");
      }
   }    
   upperComment = upperComment + "\r\n" + "Buy" + " : FallBack = $" + DoubleToStr(NormalizeDouble(fallBack,2),2);
}

void SellCross(int found)
{
   if(found == 0)
   {
      int tickSell = OrderSend(Symbol(), OP_SELL,LotSize, Bid, 3, 0, 0, TradeComment, Magic, 0, Red);
      trailing = Bid + (trailingStopWithMultiplier*Point);
      if(tickSell < 0)
      {
         Print("Order Sell Failed");
      }
   }
   upperComment = upperComment + "\r\n" + "Sell" + " : FallBack = $" + DoubleToStr(NormalizeDouble(fallBack,2),2);
}

int CloseSellOrders()
{
   int ticket, found = 0;
   double orderSize;
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true && OrderMagicNumber() == Magic)
      {
         if(OrderSymbol()== Symbol())
         {
            found++;
            ticket = OrderTicket();
            orderSize = OrderLots();
            if(OrderType() == 1)
            {
               OrderClose(ticket, orderSize, Bid, 2);
               trailing = 0.0;
               Print("Order closed due to TEMA Greater Than NoLag");
            }
         }
      }
   }
   return(found);
}

int CloseBuyOrders()
{
   int ticket, found = 0;
   double orderSize;
   for(int i=0; i<OrdersTotal(); i++)
   {
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES) == true && OrderMagicNumber() == Magic)
      {
         if(OrderSymbol()== Symbol())
         {
            found++;
            ticket = OrderTicket();
            orderSize = OrderLots();
            if(OrderType() ==0)
            {
               OrderClose(ticket, orderSize, Ask, 2);
               Print("Order closed due to TEMA Less Than NoLag");
               trailing = 0.0;
            }
         }
      }
   }
   return(found);
}